#include "base.h"
#include "poll.h"

using namespace std;

namespace Msp {
namespace IO {

Base::Base():
	mode(M_READ),
	eof_flag(false)
{ }

Base::~Base()
{
	signal_deleted.emit();
}

void Base::check_access(Mode m) const
{
	if(!(mode&m))
		throw invalid_access(m);
}

bool Base::getline(string &line)
{
	line.clear();

	if(eof_flag)
		return false;

	while(1)
	{
		int c = get();
		if(c==-1 || c=='\n')
			break;
		line += c;
	}

	return !eof_flag || !line.empty();
}

int Base::get()
{
	char c;
	if(do_read(&c, 1)==0)
		return -1;
	return static_cast<unsigned char>(c);
}

void Base::set_eof()
{
	if(!eof_flag)
	{
		eof_flag = true;
		signal_end_of_file.emit();
	}
}

const Handle &Base::get_handle(Mode)
{
	throw logic_error("Base::get_handle");
}

} // namespace IO
} // namespace Msp
